from mauve.preprocess.contractions import replace_contractions
from mauve import ENCORE
from mauve.preprocess.constants.replacements import REPLACEMENTS
from spacy.matcher import Matcher

# TODO: replace apostrophes all of below


IDIOMS = ['a bit much', 'a bite at the cherry', 'a busy bee', 'a cat has nine lives', 'a cat in gloves catches no mice', 'a cat nap', 'a cold day in july', 'a cold fish', 'a cut above', 'a cut below', 'a day late and a dollar short', 'a dog in the manger', 'a few sandwiches short of a picnic', 'a good deal', 'a great dea', 'a guinea pig', 'a hair’s breadth', 'a home bird', 'a hundred and ten percent', 'a lame duck', 'a leg up', 'a lemon', 'a life of its own', 'a little bird told me', 'a little bird told me', 'a little from column a, a little from column b', 'a lone wolf', 'a lot on one’s plate', 'a million and one', 'a notch above', 'a penny for your thoughts', 'a penny saved is a penny earned', 'a picture is worth a thousand words', 'a plum job', 'a rare bird', 'a scaredy-cat', 'a second bite at the cherry', 'a sight for sore eye', 'a sitting duck', 'a snowball’s chance in hell', 'a stitch in time saves nine', 'a stone’s throw', 'a storm in a teacup', 'a tall order', 'a week is a long time in', 'about time', 'about to', 'above and beyond', 'above board', 'above the law', 'above the salt', 'above water', 'accident of birth', 'accident waiting to happen', 'according to hoyle', 'ace in the hole', 'ace up one’s sleeve', 'acid test', 'acknowledge the corn', 'acquired taste:', 'across the board', 'across the pond', 'act high and mighty', 'act of congress', 'act one’s age', 'actions speak louder than words', 'achilles’ heel', 'add fuel to the fire', 'add insult to injury', 'add insult to injury', 'after one’s own heart', 'after the fact', 'after the lord mayor’s show (uk)', 'against the clock', 'against the clock', 'against the grain', 'against the run of play', 'age before beauty', 'agree to disagree', 'agreement in principle', 'aha moment', 'ahead of one’s time', 'ahead of the curve', 'ahead of the curve', 'ahead of the game', 'air rage', 'airy fairy', 'albatross around one’s neck', 'alive and kicking', 'all along', 'all and sundry', 'all bark and no bite', 'all bets are off', 'all dressed up and nowhere to go', 'all ears', 'all eyes and ears', 'all eyes are on', 'all fur coat and no knickers', 'all hands on deck', 'all hat and no cattle', 'all hell breaks loose', 'all in a day’s work (excl', 'all in good time', 'all in one piece', 'all it’s cracked up to be', 'all mouth and no trousers', 'all over but the shouting', 'all over hell’s half acre', 'all over the board', 'all over the map', 'all over the place', 'all rights reserved', 'all roads lead to rome', 'all set', 'all sizzle and no steak', 'all talk and no trousers', 'all told', 'all that jazz', 'all the marbles', 'all the rage', 'all the rage', 'all the same', 'all the tea in china', 'all things being equal', 'all things considered', 'all thumbs', 'all very well', 'all wet', 'along the lines of', 'amateur hour', 'amber gambler', 'amber nectar', 'american dream (the)', 'an apple a day keeps the doctor away', 'an axe', 'an early bird', 'an eye for an eye', 'an offer one can’t refuse', 'ancient history', 'and all that', 'and counting', 'and change', 'and his mother', 'and so forth', 'and so on', 'and the like', 'and then some', 'another nail in one’s coffin', 'answer back', 'ants in your pants', 'any port in a storm', 'any tom, dick or harry', 'angel’s advocate', 'angle for', 'apple of one’s eye', 'apple of someone’s eye', 'apples and oranges', 'apples and oranges', 'arm candy', 'armed to the teeth', 'around the clock', 'as american as apple pie', 'as far as i can throw (someone)', 'as fit as a fiddle', 'as pale as a ghost', 'as pale as death', 'as poor as a church mouse', 'as red as a cherry', 'asleep at the wheel (switch)', 'at death’s door', 'at each other’s throats', 'at loggerheads', 'at loggerheads', 'at sixes and sevens', 'at the drop of a hat', 'at the eleventh hour', 'at the end of one’s rope (tether)', 'at the end of the day', 'at wit’s end', 'babe in arms', 'babe in the wood', 'babe magnet', 'baby blues', 'baby boomer', 'babysitter test', 'back and forth', 'back at you', 'back burner (on the)', 'back forty', 'back in the day', 'back of beyond', 'back office', 'back on one’s feet', 'back to square one', 'back to square one', 'back to the drawing board', 'back to the salt mine(s)', 'back to the salt mines', 'back the wrong horse', 'backing and filling', 'backseat driver', 'backseat driver', 'bad apple', 'bad blood', 'bad egg', 'bad taste in one’s mouth', 'bag of tricks', 'bail out', 'ball and chain', 'ballpark figure', 'banner year', 'bang for your buck', 'bang for your buck', 'bang one’s head against the wall (against a brick wall)', 'baptism by fire', 'bar fly (or barfly)', 'bare one’s heart (soul)', 'bark up the wrong tree', 'basket case', 'bat/play for both teams', 'bat/play for the other team', 'batten down the hatches', 'batten down the hatches', 'be a barrel of laughs', 'be a cold day in hell', 'be an item', 'be footloose and fancy-free', 'be head over heels (in love)', 'be in seventh heaven', 'be in two minds (about something)', 'be like chalk and cheese', 'be lovey – dovey', 'be on the mend', 'be snowed under', 'bean counters', 'bean counters', 'beat around the bush', 'beat someone to the draw', 'beat someone to the punch', 'beat the drum for (something)', 'beauty is only skin deep', 'bed of roses', 'bedroom eyes', 'bee in one’s bonnet', 'beggar thy neighbor', 'behind the eight (or 8) ball', 'behind the scene', 'behind the times', 'bell the cat', 'bells and whistles', 'belly laugh', 'bend an elbow', 'best (greatest) thing since sliced bread', 'best of both worlds', 'bet one’s bottom dollar (on something)', 'bet the farm', 'better late than never', 'between a rock and a hard place', 'between the devil and the deep blue sea', 'beyond the pale', 'beyond the shadow of a doubt', 'big apple', 'big brother', 'big cheese', 'big deal', 'big fish', 'big picture', 'big time', 'birds of a feather', 'bird’s-eye view', 'bite off more than you can chew', 'bite the bullet', 'bite the hand that feeds you', 'bitter pill to swallow', 'black and white', 'black eye', 'black sheep', 'black-and-blue', 'blank check', 'blind date', 'blinded by love', 'blood and thunder', 'blow away the cobwebs', 'blow hot and cold', 'blow off steam', 'blow one’s top', 'blow one’s stack', 'blow the cobwebs away', 'blow the whistle', 'blow up', 'blow your own trumpet', 'blue blood', 'blue eyed boy', 'blue light special', 'bob’s your uncle', 'bolt from the blue', 'bone dry:', 'born on the wrong side of the blanket', 'borrow trouble', 'bottom of the barrel', 'boy toy', 'boys will be boys', 'brainstorm', 'break a leg', 'break out in a cold sweat', 'break the bank', 'break the ice', 'break up/ split up', 'bring home the bacon', 'bringing a knife to a gunfight', 'brush under the carpet', 'bucket list', 'bull in a china shop', 'bump in the road', 'bundle up', 'burn one’s bridges', 'burn the candle at both ends', 'burn the candle at both ends', 'burn the midnight oil', 'burn the midnight oil', 'bury (hide) one’s head in the sand', 'bury the hatchet', 'business as usual', 'busman’s holiday', 'busman’s holiday', 'busted flush', 'butter wouldn’t melt in (someone’s)', 'buy a pig in a poke', 'buy time', 'by a whisker', 'by all means', 'by hook or by crook', 'by the skin of one’s teeth', 'by word of mouth', 'call a spade a spade', 'call it a day', 'call it a night:', 'call the shots', 'call the tune', 'can’t see the forest for the trees', 'can’t swing a dead cat in', 'carrot-and-stick (approach)', 'carry a torch (for)', 'carry coals to newcastle', 'carry the can', 'cash in one’s chips', 'cash-strapped', 'cast the first stone', 'castle in the air', 'cat fight', 'cat got your tongue?', 'cat on a hot tin roof', 'cat-and-mouse', 'catch one’s death of cold', 'catch some rays', 'catch someone’s eye', 'catch-22', 'cat’s paw', 'caught red-handed', 'circle the wagons', 'claim to fame', 'clean up nicely', 'clear the air', 'clip someone’s wings', 'close, but no cigar', 'cock and bull story', 'cock-a-hoop', 'cold day in hel', 'come by something honestly', 'come clean', 'come hell or high water', 'come out in the wash', 'come out of the closet', 'come out swinging', 'come rain and shine', 'come to grips with', 'come to terms with (something)', 'coming down the pike', 'cook someone’s goose', 'cook up a storm', 'cooking up a storm', 'cool as a cucumber', 'cool cat', 'cool your heels', 'couch potato', 'crash a party', 'crickets', 'cross to bear', 'crunch time', 'crunch the numbers', 'cry over spilt (usa: spilled)', 'cry wolf', 'cry your eyes out', 'cry your eyes out', 'curiosity killed the cat', 'cut (someone) to the quick', 'cut corners', 'cut it fine', 'cut off your nose to spite your face', 'cut someone some slack', 'cut to the chase', 'cut the gordian knot', 'cut the mustard', 'cut your teeth on something', 'cutting-edge', 'champagne taste on a beer budget', 'change horses in midstream', 'change of heart', 'change one’s tune', 'changing of the guard', 'chase rainbows', 'cheap shot', 'cherry-pick', 'cherry-pick', 'chew the fat', 'chickens come home to roost', 'child’s play', 'chill out', 'chin music', 'chin up/ keep your chin up', 'chip off the old block', 'chomp (champ) at the bit', 'chomp at the bit', 'chop chop', 'chop shop', 'chuck a wobbly', 'da man (slang)', 'dance to someone’s tune', 'dance with the devil', 'dark horse', 'darken someone’s door', 'dead ahead', 'dead as the dodo', 'dead eye', 'dead heat', 'dead of winter', 'dead ringer', 'dead run', 'dead shot', 'deep pockets', 'deliver the goods', 'devil’s advocate', 'dirty look', 'do 12-ounce curls', 'dodge a bullet', 'doesn’t amount to a hill of beans', 'dog days of the summer', 'dog in the manger', 'dog-and-pony show', 'dog-eat-dog', 'don’t judge a book by its cover', 'don’t cry over spilled milk', 'don’t look a gift horse in the mouth', 'double-dip', 'double-edged sword', 'down in the dumps', 'down the road', 'drag one’s feet (or heels)', 'drag your feet', 'drain the lizard', 'draw a blank', 'draw a line in the sand', 'draw a line under (something)', 'draw a long bow', 'draw the line', 'dressed up to the nines', 'drink the kool-aid', 'drive a hard bargain', 'drive a hard bargain', 'drive a wedge between', 'drive someone up the wall', 'drop a line', 'drop the ball', 'dry run', 'dutch courage', 'dutch uncle', 'dyed-in-the-wool (adj.)', 'eager beaver', 'eagle-eyed', 'early bird', 'eat crow', 'eat humble pie', 'eat someone’s lunch', 'eat your heart out!', 'eighty-six (v.)', 'elephant in the room', 'elevator music', 'elevator pitch', 'eleventh hour', 'even steven', 'every dog has his', 'every man and his dog', 'every man for himself', 'excused boots', 'face the music', 'face the music', 'fall for something', 'fall in love with somebody', 'fall off the wagon', 'fall on one’s sword', 'fall prey to', 'fancy someone (british english)', 'farther (on) down the road', 'farther (on) down the road', 'fashion-forward', 'fat cat', 'father figure', 'feast your eyes on', 'feather in one’s cap', 'feather one’s (own) nest', 'feather one’s nest', 'fed up with', 'feel like a million dollars', 'feel on top of the world', 'fell off a truck', 'fell off the back of a lorry', 'fifteen minutes of fame', 'fifth wheel', 'fight fire with fire', 'fight like cat and dog', 'find one’s voice', 'find your feet', 'finger-pointing', 'finger-pointing', 'fire in the belly', 'first in, best dressed', 'fish for compliments', 'fish or cut bait', 'fish out of water', 'five-finger discount', 'flash in the pan', 'flat broke', 'flat out like a lizard', 'flesh and blood', 'flew the coop', 'flip-flop', 'fly by the seat of one’s pants', 'fly high', 'fly off the handle', 'fly off the handle', 'follow in someone’s footsteps', 'follow your heart', 'food for thought', 'for a song', 'for a song', 'for crying out loud (excl.)', 'for xyz reasons', 'foul play', 'fourth estate', 'fox in the henhouse (chickenhouse)', 'freak out', 'french leave', 'freudian slip', 'from pillar to post', 'from scratch', 'from soup to nuts', 'from the bottom of one’s heart', 'fubar', 'fu** (or screw) the dog (pooch)', 'full fathom five', 'full of the joys of spring', 'game of chicken', 'get a charley horse', 'get a word in edgewise', 'get along (with someone)', 'get bent out of shape', 'get carried away', 'get in on the ground floor', 'get in shape', 'get off scot free', 'get off scot free', 'get one’s ducks in a row', 'get one’s hands dirty', 'get someone’s goat', 'get to grips with', 'get the ball rolling', 'get the picture', 'get the runaround', 'get the sack, be sacked', 'get the third degree', 'get wind of', 'get with the program', 'go along', 'go ape', 'go ballistic', 'go bananas', 'go bananas', 'go belly up', 'go berserk', 'go bonkers', 'go cold turkey', 'go down in flames', 'go mental', 'go nuclear', 'go nuts', 'go off half-cocked', 'go off the deep end', 'go off the rails', 'go out on a limb', 'go pear-shaped', 'go see a man about a dog', 'go to the dogs', 'go to the mattresses', 'go the extra mile', 'go under the knife', 'go viral', 'go with the flow', 'grab (take) the bull by the horns', 'grasp (grab) at straws', 'grease monkey', 'grease the wheels', 'greasy spoon', 'green around the gills', 'green as grass', 'grind one’s teeth', 'guilty pleasure', 'guinea pig', 'give and take', 'give ’em hell (often excl.)', 'give lip service\xa0to', 'give lip service', 'give one’s two cents (that’s my two cents)', 'give someone a holler', 'give someone a piece of your mind:', 'give someone a run for their money', 'give someone an earful', 'give someone the cold shoulder', 'give someone the old heave-ho', 'give something a whirl', 'give the green light', 'hail mary (n. or adj.)', 'hair of the dog (that bit you)', 'hands are tied', 'hands down', 'hang it up', 'hang tough', 'hanging by a thread', 'happy-go-lucky', 'hard nut to crack', 'has the cat got your tongue?', 'hat trick', 'hatchet job', 'haul over the coals', 'have (one’s) head in the clouds', 'have a ball', 'have a bone to pick (with someone)', 'have a bone to pick (with someone)', 'have a chip on one’s shoulder', 'have a dog in the hunt (fight, race)', 'have a lead foot', 'have a lot on one’s plate', 'have a lot riding on (something)', 'have a nose for (something)', 'have a screw loose', 'have a tough row to hoe', 'have a whale of a time', 'have an ace up one’s sleeve', 'have bigger fish to fry', 'have egg on your face', 'have foot-in-mouth disease', 'have hand of aces/hold all the aces', 'have it out with someone', 'have one foot in the grave', 'have one over the eight', 'have one too many', 'have one’s cake and eat it, too', 'have skin in the game', 'have something in the bag', 'have the hots for (somebody', 'have the hots for somebody', 'have the time of your life', 'have the time of your life', 'have your nose in the air:', 'have your say', 'have your thumb up your ass', 'he who laughs last laughs best', 'he would put legs under a chicken', 'head (go) south', 'head and shoulders above', 'head and shoulders', 'head start', 'heads up (excl.)', 'heads up', 'heads will roll (are going to roll)', 'heads will roll (are going to roll)', 'hear (something) through the grapevine', 'heart and sou', 'heavens open', 'heavy hitter', 'helicopter parenting', 'hell for leather', 'high as a kite', 'high as a kite', 'high as a kite', 'hightail it (out of there', 'highways and byways', 'hit a wall', 'hit it out of the park', 'hit the books', 'hit the ground runnin', 'hit the hay', 'hit the jackpot', 'hit the nail on the head', 'hit the road', 'hit the roof', 'hit the roof', 'hit the sack', 'hit the spot', 'hive mind', 'hobson’s choice', 'hold one’s liquor', 'hold one’s peace', 'hold the phone', 'hold the phone', 'hold your horses', 'home away from home', 'home truths', 'honor system', 'hot mess', 'hot on the heels (of)', 'hot on the heels (of)', 'hot potato', 'i wouldn’t put it past (someone)', 'if it had been a snake, it would have bitten me', 'if the shoe fits, wear it', 'i’m all ears', 'in a fog', 'in a heartbeat', 'in a jam', 'in a new york minute', 'in a nutshell', 'in a pickle', 'in a rut', 'in broad daylight', 'in clover', 'in for a penny, in for a pound', 'in full swing', 'in his cups', 'in hot water', 'in one fell swoop', 'in one’s element', 'in someone’s wheelhous', 'in touch', 'in the blink of an eye', 'in the cards', 'in the crosshairs', 'in the dark', 'in the dark', 'in the driver’s sea', 'in the hot seat', 'in the interim', 'in the limelight, in the spotlight', 'in the long run', 'in the nick of time', 'in the pipeline', 'in the red', 'in the same boat', 'in the toilet', 'in the works', 'iron out (problems, difficulties)', 'is the pope catholic?', 'it never rains but it pours', 'it takes two to tango', 'it takes two to tango', 'it won’t fly', 'itchy feet', 'it’s a wash', 'it’s all greek to me', 'it’s no skin off my (your) nose (back)', 'it’s not over till the fat lady sings', 'it’s not rocket science', 'i’ve had it up to here', 'jack of all trades', 'jam session', 'jim crow', 'join the club (excl.)', 'jump in with both feet', 'jump on the bandwagon', 'jump on the bandwagon', 'jump the gun', 'jump the shark', 'jump the track', 'jump through hoops', 'just around the corner', 'just for the record', 'just what the doctor ordered', 'kangaroo court', 'keep (something) at bay', 'keep a stiff upper lip', 'keep an eye on', 'keep an eye peeled', 'keep it under your hat', 'keep someone at arm’s length', 'keep your nose clean', 'keep your powder dry', 'keeping one’s nose to the grindstone', 'kick ass, kick butt', 'kick the bucket', 'kick the can down the road', 'kill a fly with an elephant gun', 'kill two birds with one stone', 'kill two birds with one stone', 'kill the goose that laid the golden egg', 'kink in one’s neck', 'king of the hill', 'kiss and make up', 'kith and kin', 'knock on wood; touch wood', 'knock some sense into', 'knock someone’s socks off', 'knock up', 'knockout', 'know (something) like the back of one’s hand', 'larger than life', 'last but not least', 'laughter is the best medicine', 'learn the ropes', 'leave someone in the lurch', 'lend an ea', 'let bygones be bygones', 'let bygones be bygones', 'let off steam', 'let one’s hair down', 'let sleeping dogs lie', 'let the cat out of the bag', 'let the genie out of the bottle', 'letter of the law', 'lick one’s wounds', 'life is a bowl of cherries', 'light a fire under someone', 'light at the end of the tunnel', 'like a kid in a candy store', 'like a moth to a flame', 'like father, like son', 'like shooting', 'like taking candy from a baby', 'like two peas in a pod', 'like the cat that got the cream', 'lion’s den', 'lion’s share', 'live large', 'living in cloud cuckooland', 'living on borrowed time', 'living under a rock', 'loaded for bear', 'loan shark', 'lock horns', 'long shot', 'look the other way', 'look what the cat dragged in', 'loose cannon', 'lose it', 'lose one’s touch', 'lose touch', 'lose the thread', 'love at first sight', 'love rat', 'love someone with all of one’s heart and soul', 'lower the boom', 'low-hanging fruit', 'mad as a box of (soapy) frogs', 'mad as a hatter', 'main squeeze', 'make a break for it', 'make a mountain out of a molehill', 'make a silk purse out of a sow’s ear', 'make ends meet', 'make hay (while the sun shines)', 'make love', 'make nice:', 'make one’s mark', 'make someone’s day', 'make waves', 'man cave', 'march to the beat of your own drum', 'match made in heaven', 'may-december', 'may-december marriage', 'me time', 'meeting of the minds', 'mend fences', 'mind one’s p’s and q’s', 'miss the boat', 'monday morning quarterback', 'month of sundays', 'more fun than a barrel of monkeys', 'mother nature', 'move heaven and earth', 'move the needle', 'move up in the world', 'movers and shakers', 'much of a muchness', 'mum’s the word', 'music to my ears', 'mutton dressed up as lamb', 'my dogs are barking', 'my old man, my old lady', 'my way or the highway', 'nail-biter', 'nailing jelly/jello/pudding to a wall/tree', 'neck and neck', 'neck of the woods', 'nest egg', 'never in a million years', 'never look a gift horse in the mouth', 'new wrinkle', 'nice chunk of change', 'nickel and dime', 'nine times out of ten', 'nine-to-five job', 'nip (something) in the bud', 'no holds barred', 'no love lost between', 'no names, no pack drill', 'no names, no pack drill', 'no rhyme or reason (to)', 'no room to swing a cat', 'no shit, sherlock', 'no tree grows to the sky', 'not cut out for', 'not enough room to swing a cat', 'not give a fig', 'not have a cat in hell’s chance', 'not have a prayer', 'not know jack', 'not lift a finger', 'not mince words', 'not one’s cup of tea', 'not playing with a full deck', 'not ready for prime time', 'not sit well with (someone)', 'nothing to write home about', 'nuts and bolts', 'nutty as a fruitcake', 'pack heat', 'paddle one’s own canoe', 'page-turner', 'pain in the ass;', 'pain in the neck', 'paint the town red', 'par for the course', 'pass the buck', 'pass with flying colors', 'passing fancy', 'pay through the nose (for something)', 'peaches and cream', 'pecking order', 'pencil something in', 'penny-pinching', 'pep talk', 'perfect storm', 'pet peeve', 'pick a fight:', 'pick up the slack', 'pick up the tab', 'pie in the sky', 'piece of cake', 'pin someone down', 'pinch pennies', 'pink slip', 'pipe dream', 'piping hot', 'pipped to the post', 'pissing contest', 'play ball', 'play cat and mouse', 'play hardball', 'play it by ear', 'play it by ear', 'play the percentages', 'play the ponies', 'play with fire', 'play your cards right', 'point of no return', 'point the finger at', 'point the finger', 'poison pill', 'poison pill', 'pop one’s clogs', 'pop one’s cork', 'pop the question', 'pot calling the kettle black', 'pour (rub) salt into (on) the wound (an open wound)', 'powder keg', 'powder keg', 'powder one’s nose', 'preach to the choir, preach to the converted', 'preaching to the choir', 'pretty penny', 'price yourself out of the market', 'puddle jumper', 'pull out all the stops', 'pull strings', 'pull the plug on', 'pull yourself together', 'puppies and rainbows', 'puppy dog eyes', 'puppy love', 'pure as the driven snow', 'push the envelope', 'pushing up daisies', 'pushing up daisies', 'put a thumb on the scale', 'put down roots', 'put in one’s two cents', 'put lipstick on a pig', 'put one’s face on', 'put out feelers', 'put someone on the spot', 'put that in your pipe and smoke it', 'put the best face on (something)', 'put the brakes on', 'put the cart before the horse', 'put the cart before the horse', 'put the cat among the pigeons', 'put the genie back in the bottle', 'put the pedal to the metal', 'put up with (something)', 'put words into someone’s mouth', 'put your foot down', 'put your foot in your mouth', 'put your money where your mouth is', 'quake in one’s boots', 'quarter past', 'quarter to/of', 'queer the pitch', 'quick as a flash', 'quick-and-dirty', 'quote unquote', 'race against time', 'rain cats and dogs', 'rain cats and dogs:', 'rain on someone’s parade', 'raise (someone’s) hackles', 'raise one’s voice', 'raise red flags', 'raise the bar', 'raise the roof', 'rake (someone) over the coals', 'rake over the ashes', 'rake someone over the coals', 'rank and file', 'read between the lines', 'read the tea leaves', 'rear its ugly head (said of a problem or something unpleasant)', 'rearrange the deck chairs on the titanic', 'red flag', 'red herring', 'red meat', 'red tape', 'red-light district', 'reinvent the wheel', 'riding high', 'right as rain', 'right under (one’s) nose', 'right-hand man', 'right-hand man', 'ring a bell', 'ring a bell', 'rob peter to pay paul', 'rob the cradle', 'rob the cradle', 'rock bottom', 'rock the boat', 'roll the dice on', 'roll with the punches', 'rome wasn’t built in a day', 'rookie mistake', 'rotten to the core', 'rub (something) in someone’s face', 'rub it in', 'rub someone’s nose in (something)', 'rubber-stamp', 'rule of thumb', 'run a tight ship', 'run in the family', 'run in the family', 'run into a buzz', 'run into a buzz', 'run off at the mouth', 'run on fumes', 'run out of steam', 'run the table', 'sacred cow', 'saving grace', 'scare the living daylights out of someone', 'scorched earth', 'screw the pooch', 'school of hard knocks', 'second banana', 'second stringer', 'second wind', 'see eye to eye', 'see something out of the corner of your eye', 'seize (take) the bull by the horns', 'seize the day', 'sell (someone) a bill of goods', 'sell like hotcakes', 'selling point', 'set in stone', 'set something to music', 'set the bar (too) high', 'set the thames on fir', 'set the world on fire', 'shake the dust off your shoes (feet)', 'shape up or ship out', 'sharp as a tack', 'shell game', 'shift gears', 'shipshape and bristol fashion', 'shit a brick', 'shoot from the hip', 'shoot off one’s mouth', 'shoot oneself in the foot', 'short fuse', 'shot across the bow', 'shoulder a weight off your shoulders', 'show me an x and i’ll show you a y', 'show one’s true colors', 'show your cards', 'sick and tired of', 'sick as a dog', 'sick as a parrot', 'sight for sore eyes', 'silver bullet', 'simmer down', 'sink or swim', 'sing a different tune', 'sit on (something', 'sit tight', 'sitting duck', 'sitting pretty', 'six feet under', 'six feet under', 'six of one, a half dozen of the other', 'six ways to (from) sunday', 'slam dunk', 'sleep like a baby', 'sleep with the fishes', 'slip someone a mickey', 'slippery slope', 'slower than molasses', 'small beer', 'small fry', 'small potatoes', 'smell a rat', 'smoking gun', 'snafu', 'snake oil', 'snake oil', 'sneak peek', 'soak up the sun', 'sold on (something)', 'some eggs', 'someone’s fingerprints are all over (something)', 'something to crow', 'son of a gun', 'sore point', 'sour grapes', 'sour grapes', 'spare the rod and spoil the child', 'speak of the devil (and he shall appear)', 'speak with a plum in (one’s) mouth', 'spick and span', 'spill the beans', 'spin a yarn', 'spin one’s wheels', 'spit into the wind', 'spoiling for a fight', 'spoiling for a fight', 'square the circle', 'stab someone in the back', 'stalking horse', 'stand (someone) in good stead', 'stand on one’s own two feet', 'stand one’s ground', 'start with a clean slate', 'steal someone’s thunder', 'stem the tide', 'step up one’s game', 'step up to the plate', 'stick it to the man', 'stick your nose into something', 'sticker shock', 'stick-in-the-mud', 'sticky wicket', 'stiff-necked', 'storm in a teacup', 'stormy relationship', 'stumbling block', 'straight arrow', 'strain at a gnat and swallow a camel', 'strike a chord', 'sugar daddy', 'sure-fire', 'swan song', 'swan song', 'sweep under the carpet', 'sweep under the rug', 'sweet dreams!', 'sweeten the deal', 'sweeten the pot', 'swim against the tide', 'swim with sharks', 'swim with the fishes', 'swing for the fences', 'swing for the fences', 'sword of damocles', 'take (someone) to the cleaners', 'take a deep dive (into)', 'take a flyer', 'take a gander', 'take a hike', 'take a powder:', 'take a rain check', 'take five (ten)', 'take five', 'take it easy', 'take it easy', 'take it easy', 'take it on the chin', 'take it or leave it', 'take someone to task', 'take something with a pinch (grain) of salt', 'take the cake', 'take the edge off (of something', 'take the fifth', 'take the gloves off', 'take the high road', 'take the mickey (piss) (out of someone)', 'take the shine off (something)', 'take the starch out of (someone)', 'take the wind out of someone’s sails', 'take your life in your hands', 'take your medicine', 'take your time', 'taste of your own medicine', 'teach an old dog new tricks', 'tear one’s hair out', 'tear-jerker', 'tee many martoonies', 'tell it to the marines', 'tempest in a teapot', 'ten a penny', 'ten to one', 'test the waters', 'test the waters', 'tie the knot', 'tighten the screws', 'tight-lipped', 'til the cows come home', 'time is money', 'tip of the iceberg', 'tip one’s hand', 'tlc', 'to be a peach', 'to be smitten with someone', 'to be someone’s one and only', 'to be the apple of someone’s eye', 'to bear fruit', 'to bear fruit', 'to carpool', 'to each his own', 'to get cold feet', 'to have a chip on one’s shoulder', 'to have butterflies in your stomach', 'to have one for the road', 'to pay an arm and a leg', 'to pop (one’s) cherry', 'to pull someone’s leg', 'to run hot and cold', 'to the letter', 'toe the line', 'tone-deaf', 'tongue-in-cheek', 'too busy fighting alligators to drain the swamp', 'too many cooks spoil the broth', 'too many chiefs and not enough indians', 'too many to shake a stick at', 'toot your own horn', 'top banana', 'toss a wrench (spanner) into', 'touch base', 'touch one’s heart', 'touch water', 'tough cookie', 'tough cookie', 'tough sledding', 'turn a blind eye', 'turn on a dime', 'turn someone inside out', 'turn something on its head', 'turn turtle', 'turn the corner', 'turn the tables', 'turnabout is fair play', 'twenty-four seven', 'twist the knife (in deeper)', 'twist the knife (in deeper)', 'two a penny', 'two peas in a pod', 'thank god it’s friday (tgif)', 'that ship has sailed', 'that’s music to my ears', 'that’s a stretch', 'that’s all she wrote', 'the apple never falls far from the tree', 'the birds and the bees', 'the cat is out of the bag', 'the coast is clear', 'the cherry on the cake', 'the deck is (the cards are)', 'the jig is up', 'the more things change, the more they stay the same', 'the only game in town', 'the powers that be', 'the real mccoy', 'the story has legs', 'the time is ripe', 'the walls have ears we', 'the walls have ears', 'the whole enchilada', 'the whole shebang', 'the world is your oyster', 'there but for the grace of god go i', 'there’s more than one way to skin a cat', 'there’s no such thing as a free lunch', 'thin on the ground', 'think big', 'think outside the box', 'think tank', 'think tank', 'third rail', 'third time’s a charm', 'thirty-thousand-foot view', 'this has (person x) written all over it', 'this is not your father’s', 'three sheets to the wind', 'through the grapevine', 'through thick and thin', 'throw a wet blanket on (something)', 'throw a wrench int', 'throw caution to the wind', 'throw down the gauntlet', 'throw elbows', 'throw in the towel', 'throw someone for a loop', 'throw someone under the bus', 'throw the baby out with the bath water', 'throw the baby out with the bath water', 'throw the book at', 'throw the fight', 'throw the game', 'throw the match', 'thumbs-up', 'train wreck', 'trash talk', 'tread water', 'trial balloon', 'trip the light fantastic', 'u turn', 'ugly duckling', 'under (below) the radar', 'under someone’s spell', 'under the impression', 'under the table', 'under the weather', 'under the weather', 'under wraps', 'university of life', 'until the cows come home', 'until you’re blue in the face', 'up a creek', 'up for grabs', 'up for grabs', 'up in arms', 'up in the air', 'up to one’s neck', 'up to scratch', 'up to snuff', 'up the ante', 'up the duff', 'upset the apple cart', 'use one’s head', 'vale of tears', 'vicious circle', 'victory lap', 'virgin territory', 'vote with one’s feet', 'waiting in the wings', 'waka-jumping', 'wake up and smell the coffee', 'wake up on the wrong side of the bed', 'walk on eggshells', 'walk the plank', 'wandering eye', 'wandering eye', 'wash your hands of (something)', 'water under the bridge', 'watering hole', 'weekend warrior', 'we’ll cross that bridge', 'welsh (welch) on a deal', 'wet behind the ears', 'wet behind the ears', 'wet blanket', 'wet your whistle:', 'what do you make of (him)?', 'what goes around comes around', 'what’s good for the goose is good for the gander', 'when hell freezes over', 'when in doubt, leave it ou', 'when in rome, do as the romans do', 'when it rains, it pours', 'when pigs fly', 'when the chips are down', 'where (when) the rubber', 'where there’s a will, there’s a way', 'where there’s smoke, there’s fire', 'whisper sweet nothings', 'whistle in the dark', 'whistle past the graveyard', 'whistling dixie', 'white elephant', 'who’s she, the cat’s mother?', 'wild goose chase', 'window dressing', 'window shop', 'witch hunt', 'with bells on', 'work one’s fingers to the bone', 'worn to a frazzle', 'wouldn’t be caught dead', 'writing (handwriting) on the wall', 'year in, year out', 'you can lead a horse to water, but you can’t make it drink', 'you can say that again!', 'you can take it to the bank', 'you can’t judge a book by its cover', 'you can’t make an omelet (omelette)', 'you can’t make fish of one and fowl of the other', 'you know the drill', 'you snooze, you lose', 'young at heart', 'your guess is as good as mine', 'your mileage may vary', 'your number is up', 'you’re driving me nuts', 'yours truly', 'zero in on', 'zig when one should be zagging', 'zip one’s lip']

IDIOMS.extend(['a little learning is a dangerous thing', 'a snowball effect', "a snowball's chance in hell", 'a stitch in time saves nine', 'a storm in a teacup', 'an apple a day keeps the doctor away', 'an ounce of prevention is worth a pound of cure', 'as right as rain', 'bolt from the blue', 'burn bridges', 'calm before the storm', 'come rain or shine', 'curiosity killed the cat', 'cut the mustard', "don't beat a dead horse", 'every dog has his day', 'familiarity breeds contempt', 'fit as a fiddle', 'fortune favours the bold', 'get a second wind', 'get wind of something', 'go down in flames', 'haste makes waste', 'have your head in the clouds', 'he who laughs last laughs loudest', "hear something straight from the horse's mouth", "he's not playing with a full deck", "he's off his rocker", "he's sitting on the fence", 'it is a poor workman who blames his tools', 'it is always darkest before the dawn', 'it takes two to tango', 'jump on the bandwagon', 'know which way the wind is blowing', 'leave no stone unturned', 'let sleeping dogs lie', 'like riding a bicycle', 'like two peas in a pod', 'make hay while the sun shines', 'on cloud nine', 'once bitten, twice shy', 'out of the frying pan and into the fire', 'run like the wind', 'shape up or ship out', 'snowed under', 'that ship has sailed', 'the pot calling the kettle black', 'there are clouds on the horizon', "those who live in glass houses shouldn't throw stones", 'through thick and thin', 'time is money', 'waste not, want not', 'we see eye to eye', 'weather the storm', 'well begun is half done', 'when it rains it pours', 'you can catch more flies with honey than you can with vinegar', "you can lead a horse to water, but you can't make him drink", "you can't make an omelet without breaking some eggs"])


IDIOMS.extend(['a bird in the hand is worth two in the bush', 'a bunch of fives', 'a chain is only as strong as its weakest link', 'a change is as good as a rest', 'a countenance more in sorrow than in anger', 'a daniel come to judgement', 'a diamond in the rough', 'a diamond is forever', 'a different kettle of fish', 'a dish fit for the gods', "a dog is a man's best friend", 'a drop in the bucket', 'a drop in the ocean', 'a fate worse than death', "a feather in one's cap", 'a fish rots from the head down', 'a fish out of water', 'a fly in the ointment', 'a fool and his money are soon parted', "a fool's paradise", 'a foot in the door', 'a foregone conclusion', 'a friend in need is a friend indeed', 'a golden key can open any door', 'a good man is hard to find', 'a hard man is good to find', 'a hiding to nothing', 'a horse, a horse, my kingdom for a horse', 'a house divided against itself cannot stand', 'a journey of a thousand miles begins with a single step', 'a knight in shining armour', 'à la carte', 'à la mode', "a legend in one's own lifetime", 'a leopard cannot change its spots', 'a little bird told me', 'a little knowledge is a dangerous thing', 'a little of what you fancy does you good', 'a load of cobblers', 'a load of codswallop', 'a man after my own heart', 'a man who is his own lawyer has a fool for a client', "a millstone around one's neck", 'a ministering angel shall my sister be', 'a miss is as good as a mile', 'a nation of shopkeepers', 'a nest of vipers', 'a no-brainer', 'a nod is as good as a wink', 'a norange', 'a penny for your thoughts', 'a picture is worth a thousand words', 'a piece of the action', 'a pig in a poke', 'a place for everything and everything in its place', 'a plague on both your houses', 'a priori', 'a red rag to a bull', 'a riddle wrapped up in an enigma', 'a rolling stone gathers no moss', 'a rose by any other name would smell as sweet', 'a rose is a rose is a rose', 'a safe pair of hands', 'a sea change', 'a skeleton in the closet', 'a sledgehammer to crack a nut', 'a shot in the arm', 'a sight for sore eyes', 'a sorry sight', 'a stitch in time saves nine', "a stone's throw", 'a thing of beauty is a joy forever', "a tinker's damn", 'a turn up for the books', 'a watched pot never boils', "a wolf in sheep's clothing", 'a woman needs a man like a fish needs a bicycle', "a woman's place is in the home", 'a word in edgeways', 'a word in your shell-like', "a young man's fancy", 'a1 at lloyds', 'abandon all hope ye who enter here', 'abide with me', 'about face', 'about turn', 'above board', 'abracadabra', 'absence makes the heart grow fonder', 'absent without leave', 'absolute power corrupts absolutely', 'accidentally on purpose', 'accidents will happen', 'according to hoyle', 'ace in the hole', "achilles' heel", 'acid test', 'acronyms', 'across the board', 'act of god', 'act the giddy goat', 'action man', 'actions speak louder than words', "adam's ale", 'adversity makes strange bedfellows', 'after the fact', 'aga saga', 'against the grain', 'age before beauty', 'age cannot wither her, nor custom stale her infinite variety', 'agree to disagree', 'aid and abet', 'aide-mémoire', 'air kiss', 'air quotes', 'al fresco', 'alas, poor yorick! i knew him, horatio', 'alike as two peas in a pod', 'alive and kicking', 'all agog', 'all at sea', 'all fingers and thumbs', 'all greek to me', 'all in all', 'all intents and purposes', 'all mouth and no trousers', 'all of a sudden', 'all present and correct', 'all publicity is good publicity', 'all singing, all dancing', 'all systems go', 'all that glitters is not gold / all that glisters is not gold', 'all the tea in china', 'all things come to those who wait', 'all things must pass', 'all you can eat', "all's fair in love and war", 'all work and no play makes jack a dull boy', 'alter ego', 'amber nectar', "an albatross around one's neck", 'an apple a day keeps the doctor away', 'an arm and a leg', 'an axe to grind', "an englishman's home is his castle", 'an eye for an eye, a tooth for a tooth', 'an ill wind', "an offer he can't refuse", 'an oxford scholar', 'anchors aweigh', 'angry young man', 'ankle biter', 'annus horribilis', 'another day, another dollar', 'another think coming', 'apple of my eye', 'apple pie order', 'april fool', 'ear candy', 'arms akimbo', 'ars longa, vita brevis', 'arty-farty', 'asap - as soon as possible', 'as x as y', 'as alike as two peas in a pod', 'as bald as a coot', "as black as newgate's knocker", 'as bold as brass', 'as brown as a berry', 'as busy as a bee', 'as busy as a one-armed paperhanger', 'as cold as any stone', 'as cool as a cucumber', "as cute as a bug's ear", 'as daft as a brush', 'as dead as a dodo', 'as dead as a doornail', 'as different as chalk and cheese', 'as easy as pie', 'as easy as taking candy from a baby', 'as fast as greased lightning', "as fine as frog's hair", "as fit as a butcher's dog", 'as fit as a fiddle', 'as good as gold', 'as good luck would have it', 'as happy as...', 'as happy as a clam', 'as happy as a sandboy', 'as happy as larry', 'as high as a kite', 'as keen as mustard', 'as mad as a hatter', 'as mad as a march hare', 'as nice as ninepence', 'as old as methuselah', 'as old as the hills', 'as pleased as punch', 'as pure as the driven snow', 'as queer as a nine bob note', 'as safe as houses', 'as sick as a parrot', 'as snug as a bug in a rug', 'as straight as a die', 'as the crow flies', 'as thick as thieves', 'as thick as two short planks', 'as white as snow', 'ashes to ashes dust to dust', "ask a silly question and you'll get a silly answer", 'for whom the bell tolls', 'asta la vista, baby', 'at loggerheads', 'at one fell swoop', "at one's beck and call", "at one's wit's end", 'at sixes and sevens', 'attack is the best form of defence', 'augur well', 'auld lang syne', 'away with the fairies', 'baby blues', 'baby boomer', 'baby father', 'back of beyond', 'back-seat driver', 'back the field', 'back to basics', 'back to square one', 'back to the drawing board', 'backroom boy', 'backward in coming forward', 'bacon', 'bad books', 'bad egg', 'bad hair day', 'bad money drives out good', 'badger to death', 'bag and baggage', "baker's dozen", 'balance of power', 'balance of trade', 'bald as a coot', 'bale out/bail out', 'ball and chain', 'ballpark figure', 'bandy words', 'bane of your life', 'bang on about', 'baptism of fire', 'barge-pole', 'barking mad', 'barking up the wrong tree', 'barrel of laughs', 'basket case', 'bated breath', 'bats in the belfry', 'batten down the hatches', 'battle royal', 'be afraid, be very afraid', 'be enthralled', 'be still, my beating heart', 'beam ends', 'bean counter', 'beast with two backs', 'beat a hasty retreat', 'beat around the bush', 'beat swords into ploughshares', 'beat the living daylights out of someone', 'beauty is in the eye of the beholder', 'beauty is only skin deep', 'beck and call', 'bed of roses', 'bee in your bonnet', 'beef and reef', 'beelzebub has a devil for a sideboard', 'been there, done that', 'beer and skittles', 'beer goggles', 'bee-line', "bee's knees", 'beetle-browed', 'before the fact', 'before you can say jack robinson', 'beg the question', 'beggar belief', "beggars can't be choosers", "behind every great man there's a great woman", 'behind the eight ball', 'bell, book and candle', 'bell the cat', 'belle of the ball', 'below the belt', 'below the salt', 'bells and whistles', 'belt and braces', 'belt up', 'best bib and tucker', 'best laid schemes of mice and men', 'bet your bottom dollar', 'better half', 'better late than never', 'better to have loved and lost than never to have loved at all', 'between a rock and a hard place', 'between the devil and the deep blue sea', 'between two stools', 'between you, me and the bed-post', 'beware of greeks bearing gifts', 'beware the ides of march', 'beyond a shadow of a doubt', 'beyond belief', 'beyond our ken', 'beyond the pale', 'biblical phrases', 'big apple', 'big cheese', 'big easy', 'big fish in a small pond', "big fleas have little fleas upon their back to bite 'em", 'big wig', 'bigger bang for your buck', 'bill stickers is innocent', 'billy-o', 'billy no mates', 'binge drinking', 'bird in the hand is worth two in the bush', 'birds and the bees', 'birds of a feather flock together', 'bite the bullet', 'bite the dust', 'bitter end', "black as newgate's knocker", 'black-on-black', 'black sheep of the family', 'blast from the past', 'blaze a trail', 'bless you!', 'blind leading the blind', "blind-man's buff", 'bling-bling', 'blonde bombshell', 'blood and thunder', 'blood is thicker than water', 'blood toil tears and sweat', 'blood, sweat and tears', 'bloody-minded', 'blot on the landscape', 'raspberry tart', 'blow the gaff', 'blow your mind', 'blow your own trumpet', 'blown to smithereens', 'blue blood', 'blue moon', 'blue-plate special', "bob's your uncle", 'bode well', 'bodice ripper', 'body - phrases related to the human body', 'body surfing', 'bog standard', 'bold as brass', 'bolt from the blue', 'bone dry', 'bone idle', 'bone up on', 'booby prize', 'booby trap', 'boogie-woogie', 'boots', 'booze cruise', 'born again', "born with a silver spoon in one's mouth", 'born within the sound of bow bells', 'boss eyed', 'bottle out', 'bottom drawer', 'bottom-up', 'bought the farm', 'bowled a maiden over', 'box and cox', 'box-seat', 'boxing day', 'boys will be boys', 'brand spanking new', 'brass monkey weather', 'brass tacks', 'bread always falls buttered side down', 'bread of life', 'break a leg', 'break the ice', 'bricks and clicks', 'bring home the bacon', 'broad in the beam', 'broke', 'brook no truck with', 'bronx cheer', 'brown as a berry', 'browned off', 'brownie points', 'brummagem screwdriver', 'brush', 'bubble and squeak', 'buck stops here', 'bucket list', 'buckle down', 'build a better mousetrap and the world will beat a path to your door', 'bugger bognor!', "buggins' turn", 'bullet', 'bums on seats', 'bun in the oven', 'bunch of fives', 'bunny boiler', 'burn the candle at both ends', 'burning the midnight oil', 'bury the hatchet', 'bury your head in the sand', 'busy as a bee', 'butt end of the stick', 'butter no parsnips', 'butter side down', "butter wouldn't melt in his mouth", 'butterfingers', 'by and large', 'by dint of', 'by gum', 'by hook or by crook', 'by the board', 'by the book', 'by the short hairs', 'by the skin of your teeth', 'by your leave', 'bygones be bygones', 'caesar salad', 'call a spade a spade', 'call of the wild', 'camera cannot lie', "can't be overestimated/can't be underestimated", "can't hold a candle to", 'canteen culture', 'canterbury pace', 'captain francis grose', 'carbon-copy', 'carbon footprint', 'card-sharp', 'carey street - on', 'carpe diem', 'carry coals to newcastle', 'cart before the horse', 'carte blanche', 'cash on the nail', 'cast a clout till may be out', 'cast the first stone', 'cat burglar', 'cat got your tongue?', 'cat may look at a king', 'cat out of the bag', 'catbird seat - in the', 'catch a crab', 'catch 22', "cat's cradle", "cat's pajamas", 'caught in a cleft stick', 'caught by the short hairs', 'caught red-handed', 'chain is only as strong as its weakest link', 'chaise lounge', 'chalk and cheese', 'champ at the bit', 'chance would be a fine thing', 'channel surfing', 'charity begins at home', 'charley horse', 'charm offensive', 'charmed life', 'cheap at half the price', 'cheek by jowl', 'cherchez la femme', 'chew the cud', 'chick flick', 'chickens come home to roost', 'children should be seen and not heard', 'chinese whispers', 'chinless wonder', 'chip off the old block', 'chip on your shoulder', 'chit-chat', 'chock-a-block', 'chop and change', 'chop-chop', 'chow down', 'christmas box', 'christmas card verses', 'clean breast', 'cleanliness is next to godliness', 'clear blue water', 'cleft stick', 'cliff-hanger', 'climb on the bandwagon', 'cloak and dagger', 'clod-hopper', 'clog up', 'close, but no cigar', 'close quarters', 'close your eyes and think of england', 'cloud cuckoo land', 'cloud nine', 'club sandwich', 'clue', 'clutch at straws', 'coals to newcastle', 'coat of many colours', 'cobblers', 'cock-a-hoop', 'cock-sure', 'cock a snook', 'cock and bull story', 'cock-up', 'cockney rhyming slang', 'codswallop', 'cogito ergo sum', 'coin a phrase', 'cold as any stone', 'cold comfort', 'cold enough to freeze the balls off a brass monkey', 'cold feet', 'cold shoulder', 'cold turkey', "colder than a witch's tit", 'collywobbles', 'colour up', 'come a cropper', 'come clean', 'come on down', 'come up trumps', 'come what come may', 'comes to the crunch', 'on a wing and a prayer', 'common sense', 'comparisons are odious', 'compassion fatigue', 'complete shambles', 'concrete overcoat', 'cook the books', 'cookie cutter', 'cool as a cucumber', 'cool britannia', 'cop an attitude', 'copper-bottomed', 'cor blimey', 'cordon bleu', 'cost an arm and a leg', 'cotton on to', 'cotton-picking', 'count your chickens before they are hatched', 'countenance more in sorrow than in anger', 'counting sheep', 'country bumpkin', 'course of true love never did run smooth', 'crack of doom', 'crackpot', 'crapper', 'crocodile tears', 'crop up', 'crown jewels', 'cry havoc and let slip the dogs of war', "curate's egg", 'curiosity killed the cat', 'curry favour', 'customer is always right', 'cut and run', 'cut of your jib', 'cut off without a penny', 'cut off your nose to spite your face', 'cut the mustard', 'cut to the chase', "cute as a bug's ear", 'daft as a brush', 'damp squib', 'dance attendance on', 'daniel come to judgement', 'darby and joan', 'dark horse', 'darkest hour', 'darkest hour is just before the dawn', 'dark side', 'darling buds of may', 'dash to pieces', 'date rape', "davy jones' locker", 'daylight robbery', 'dead as a dodo', 'dead as a doornail', 'dead cat bounce', 'dead in the water', 'dead ringer', 'delusions of grandeur', 'derring-do', 'designer stubble', 'deus ex machina', 'devil and the deep blue sea', 'devil incarnate', 'devil take the hindmost', 'devil to pay', "devil's advocate", 'diamond in the rough', 'diamond is forever', 'dicky-bird', 'die-hard', 'die has been cast', 'different as chalk and cheese', 'different kettle of fish', 'differently abled', 'dirt bag', 'dish fit for the gods', 'do unto others as you would have them do to you', 'dock your pay', "doesn't know shit from shinola", 'doff your hat', 'dog days', 'dog in the manger', "dog is a man's best friend", "dog's bollocks", "dog's breakfast", "dog's dinner", 'doldrums', 'dollars to doughnuts', 'done a runner', 'done to a turn', "donkey's years", "don't call us, we'll call you", "don't cast your pearls before swine", "don't change horses in midstream", "don't count your chickens before they are hatched", "don't die like i did", "don't get mad, get even", "don't go there", "don't have a clue", "don't keep a dog and bark yourself", "don't know shit from shinola", "don't let the bastards grind you down", "don't look a gift horse in the mouth", "don't shut the stable door after the horse has bolted", "don't throw good money after bad", "don't throw the baby out with the bathwater", "don't try to teach your grandma to suck eggs", 'doom and gloom', 'double cross', 'double, double toil and trouble, fire burn, and cauldron bubble', 'double dutch', 'double entendre', 'double standard', 'double whammy', 'down at heel', 'down in the dumps', 'down to the wire', 'down the pan', 'down the tubes', 'drag race', 'dragged through a hedge backwards', 'draw a blank', 'draw your horns in', 'dreams of empire', 'dressed to the nines', 'drink like a fish', 'driving while black', 'drop-dead gorgeous', 'drop in the bucket', 'dropping like flies', 'drummed out of the army', 'ducks and drakes', 'dumb blonde', 'dust up', 'dutch', 'dutch courage', 'duvet day', 'ear candy', 'early bird catches the worm', 'early to bed and early to rise makes a man healthy, wealthy and wise', 'ear-mark', "east, west, home's best", 'easy as pie', 'eat drink and be merry', 'eat humble pie', 'eat my hat', 'eaten out of house and home', 'economical with the truth', 'eeny, meeny, miny, mo', 'egg on', 'eighty six', 'elbow-grease', 'elementary my dear watson', 'elephant in the room', 'elvis has left the building', "emperor's new clothes", 'end of story', 'ends of the earth', 'enough is as good as a feast', 'enough is enough', "englishman's home is his castle", 'eponymous phrases', 'et tu, brute', 'ethnic cleansing', 'euphemisms', 'even at the turning of the tide', 'every cloud has a silver lining', 'everybody out', 'exceedingly well read', 'exception that proves the rule', 'excuse me while i kiss this guy', 'excuse my french', 'extraordinary rendition', 'ear candy', 'eye for an eye, a tooth for a tooth', 'eye of newt and toe of frog, wool of bat and tongue of dog', 'face that launched a thousand ships', 'face the music', 'faff about', 'fag end', 'failing to plan is planning to fail', 'faint-hearted', 'fair and square', 'fair dinkum', 'fair exchange is no robbery', 'fair play', 'fair to middling', 'fairy-tale ending', 'fait accompli', 'faith will move mountains', 'fall from grace', 'fall guy', 'fall off the back of a lorry', 'fall on your sword', 'famous for fifteen minutes', 'famous last words', 'famous last words', 'fancy free', 'fancy pants', 'fanny adams', 'far be it from me', 'far from the madding crowd', 'farmers', 'fashion victim', 'fast', 'fast and loose', 'fast asleep', 'fathom out', 'fat of the land', 'fate worse than death', "feather in one's cap", 'fed up', 'feeding frenzy', 'fell off the back of a truck', 'fell swoop', 'fellow traveller', 'female of the species is more deadly then the male', 'fend for yourself', 'fend off', 'fiddlesticks', 'fiddling while rome burns', 'fie, foh, fum, i smell the blood of an englishman', 'field day', 'fifteen minutes of fame', 'fifth column', 'fight fire with fire', 'fight the good fight', 'filthy rich', "finagle's law", "fine as frog's hair", 'fine words butter no parsnips', 'finger lickin good', 'fingers and thumbs', 'first dibs', 'first water', 'first world', 'fish rot from the head down', 'fish or cut bait', 'fish out of water', "as fit as a butcher's dog", 'fit as a fiddle', 'fifth column', 'fifth estate', 'fits and starts', 'fits to a tee', "five o'clock shadow", 'fixer-upper', 'flash in the pan', 'flat out', 'flavor of the month', 'flesh and blood', 'flogging a dead horse', 'floozie in the jacuzzi', 'flotsam and jetsam', 'fly by night', "fly by the seat of one's pants", 'fly in the ointment', 'fly off the handle', 'fly on the wall', 'foam at the mouth', 'fobbed off', 'folk etymology', 'fool and his money are soon parted', 'fools rush in where angels fear to tread', "fool's errand", "fool's gold", "fool's paradise", 'foot in the door', 'foot of our stairs', 'for all intents and purposes', 'for crying out loud', 'for ever and a day', 'for every thing there is a season', 'for good measure', 'for keeps', 'for the birds', 'for whom the bell tolls', 'forbidden fruit', 'foregone conclusion', 'forewarned is forearmed', 'forgive them for they know not what they do', 'fork out', 'forlorn hope', 'foul play', 'four by two', 'four corners of the earth', 'frailty, thy name is woman', 'francis grose', 'freeze the balls off a brass monkey', 'freezing temperatures', 'french phrases', 'friday afternoon car', 'friend in need is a friend indeed', 'friends, romans, countrymen, lend me your ears', 'frog in the throat', "frog's hair", 'from pillar to post', 'from sea to shining sea', 'from strength to strength', 'fruits of your loins', 'fuddy-duddy', 'full monty', 'full of piss and vinegar', 'full tilt', 'full to the gunwales', 'funny farm', 'fuzzy wuzzy', 'gad zooks', 'game is afoot', 'game is up', 'game of two halves', 'gee whiz', 'generation x', 'genius is one percent inspiration and 99 percent perspiration', 'get a word in edgeways', 'get down to brass tacks', 'get medieval', 'get off on the wrong foot', 'get off your high horse', 'get on my wick', "get one's dander up", 'get over it', 'get the pip', 'get the sack', 'get the upper hand', 'get thee behind me satan', 'get underway', 'get used to it', 'get your dander up', 'get your goat', 'giddy aunt', 'giddy goat', 'gild the lily', 'gilt off the gingerbread', 'ginned up', 'ginger', 'ginger up', 'give a man a fish and you will feed him for a day', 'give a wide berth', 'give no quarter', 'give the devil his due', 'give up the ghost', 'giving me gyp', 'glass ceiling', 'gloom and doom', 'go and boil your head', 'go berserk', 'go by the board', 'go by the book', 'go dolally', 'go-faster', 'go for a burton', 'go haywire', 'go like the clappers', 'go off half-cocked', 'go out on a limb', 'go postal', 'go over like a lead balloon', 'go pound sand', 'go the whole hog', 'go to hell in a handbasket', 'go to pot', 'go to the dogs', 'go to the foot of our stairs', 'go to the mattresses', 'god bless you!', 'god is dead', 'god rot them', 'godfrey daniel', 'going for a burton', 'golden key can open any door', 'golf - gentlemen only, ladies forbidden', 'gone for a burton', 'good as gold', 'good in parts', 'good man is hard to find', 'good measure', 'good men and true', 'good money after bad', 'good riddance', 'good samaritan', 'good things come to those that wait', 'goodnight vienna', 'goody, goody gumdrops', 'goody two-shoes', 'gordon bennett', 'got my mojo working', 'grace of god', 'grain of salt', 'grand slam', 'grandfather clock', 'granny dumping', 'grasp the nettle', 'grass up', 'graveyard shift', 'greased lightning', 'great balls of fire', 'great minds think alike', 'great unwashed', 'green eyed monster', 'gregory peck', 'gridlock', 'grind to a halt', 'grinning like a cheshire cat', 'grist to the mill', 'grose', 'gry - words ending in', 'guinea pig', 'gung ho', 'gussied-up', 'guts for garters', 'hair of the dog that bit you', 'hairy eyeball', 'halcyon days', 'half a loaf is better than no bread', 'half-cocked', 'half-hearted', 'half inch', 'hand over fist', 'handbags at ten paces', 'handle with kid gloves', 'hands down', 'hanged, drawn and quartered', 'hanky-panky', 'happy as a clam', 'happy as a sandboy', 'happy as larry', 'happy clappy', 'happy slapping', 'happily ever after', 'harbinger of doom', 'hard and fast', 'hard cases make bad law', 'hard cheese', 'hard hearted', 'hard lines', 'hard man is good to find', "hark, hark! the lark at heaven's gate sings", 'harp on', 'harvest moon', 'has the cat got your tongue?', 'hasta la vista, baby', 'haste makes waste', 'hat trick', 'have a captain cook', 'have a dekko', 'have an axe to grind', 'have an inkling', 'have no truck with', "haven't got a clue", 'have your guts for garters', 'have your work cut out', 'he who can, does; he who cannot, teaches', 'he who laughs last laughs longest', 'he who sups with the devil should have a long spoon', 'he will give the devil his due', 'head over heels', 'heads up', 'heads will roll', 'hear hear', 'heard it through the grapevine', "heart's content", 'heavens to betsy', 'heavens to murgatroyd', 'heavy metal', 'heebie-jeebies', 'hedge your bets', 'hell in a handbasket', 'hell has no fury like a woman scorned', 'hell or high water', "hell's bells", 'helter-skelter', 'hem and haw', 'here lies one whose name is writ in water', 'het up', 'hiding to nothing', 'higgledy-piggledy', 'high and dry', 'high and mighty', 'high as a kite', 'high five', 'high-flyer', 'high horse', 'high on the hog', 'high time', 'high, wide and handsome', 'hissy fit', 'history is bunk', 'hit for six', 'hit the ground running', 'hit the hay', 'hit the nail on the head', 'hither and yon', "hobson's choice", 'hobby-horse', 'hocus pocus', 'hoi polloi', 'hoist with your own petard', 'hoity-toity', 'hold a candle to', 'hold your horses', 'home is where the heart is', 'homonyms', 'honesty is the best policy', 'honey catches more flies than vinegar', 'hooray henry', 'horse and cart', 'horse feathers', 'horse-sense', 'horse, a horse, my kingdom for a horse', "horse's mouth", 'hot-blooded', 'hot off the press', 'hot on the heels', 'house divided against itself cannot stand', 'household words', 'houston, we have a problem', 'how are the mighty fallen?', 'how do you do?', 'how now brown cow?', "how sharper than a serpent's tooth it is to have a thankless child", 'hugger-mugger', 'hung, drawn and quartered', 'hunky-dory', 'hunt and peck', 'hush puppies', 'i bear a charmed life', 'i have not slept one wink', 'i have nothing to declare but my genius', 'i have nothing to offer but blood toil tears and sweat', "i haven't got a clue", 'i see you stand like greyhounds in the slips', 'i spy with my little eye', 'i told you is was ill', 'i will swing for you', 'i will wear my heart upon my sleeve', 'identity theft', 'idioms quiz', 'ides of march', 'if i had my druthers', "if it ain't broke, don't fix it", 'if music be the food of love, play on', 'if the cap fits', 'if the mountain will not come to muhammad, then muhammad must go to the mountain', 'if the shoe fits', 'if wishes were horses, beggars would ride', "if you can't stand the heat, get out of the kitchen", 'if you think that, you have another think coming', 'ill wind', "i'll be there with bells on", "i'll go to the foot of our stairs", "i'll have your guts for garters", "i'm stumped", 'imitation is the sincerest form of flattery', 'in a cleft stick', 'in a pickle', "in a pig's eye", 'in a quandary', 'in a nutshell', 'in a trice', 'in an interesting condition', 'in for a penny, in for a pound', 'in like flynn', 'in limbo', 'in my minds eye', "in someone's bad books", 'in spades', 'in stitches', 'in the bag', 'in the box-seat', 'in the buff', 'in the cart', 'in the catbird seat', 'in the club', 'in the doghouse', 'in the doldrums', 'in the limelight', 'in the nick of time', 'in the pink', 'in the red', 'in the same boat', 'in the sticks', 'in your face', 'including, but not limited to', 'indian giver', 'indian summer', 'infra dig', 'innocent until proven guilty', 'inside out', 'intents and purposes', 'iron hoof', 'is the pope catholic?', 'is the pope polish?', 'is this a dagger which i see before me?', "it ain't over till the fat lady sings", 'it came like a bolt from the blue', 'it is meat and drink to me', 'it never rains but it pours', "it's all greek to me", "it's all grist to the mill", "it's better to give than to receive", "it's better to have loved and lost than never to have loved at all", "it's better to light a candle than curse the darkness", "it's not rocket science", "it's that man again", 'ivory tower', 'jack in the box', 'jack of all trades, master of none', 'jack palancing', 'jack - phrases that include the name jack', 'jack robinson', 'jack tar', 'jack the lad', 'jam tomorrow', 'jerry built', 'jet-black', 'jimmy horner', 'jimmy riddle', 'jobs for the boys', 'join the colours', 'joined at the hip', 'johnny on the spot', 'jot or tittle', 'jump on the bandwagon', 'jump the gun', 'jump the shark', 'jury is still out', 'just deserts', 'just in time', 'kangaroo court', 'katy bar the door', 'keel over', 'keep a wide berth', 'keen as mustard', 'keep body and soul together', 'keep schtum', 'keep a stiff upper lip', 'keep at bay', 'keep it under your hat', 'keep the ball rolling', 'keep up with the joneses', 'keep your chin up', 'keep your distance', 'keep your hands clean', 'keep your nose clean', 'keep your nose to the grindstone', 'keep your pecker up', 'keep your powder dry', 'keeping up with the joneses', 'keepy-uppy', 'kettle of fish', 'khyber pass', 'kick the bucket', 'kick your heels', "kickin'", 'kilroy was here', "king james' version of the bible", 'kiss and tell', 'kiss me hardy', 'kit and caboodle', 'kith and kin', 'knee-jerk reaction', 'knick-knack', 'knight in shining armour', 'knock back', 'knock into a cocked hat', 'knock off', 'knock on wood', 'know the ropes', 'know your onions', 'know which side your bread is buttered', 'know which way the wind blows', 'knuckle down', 'knuckle under', 'kowtow to', 'la-di-da', 'la dolce vita', 'labour of love', 'lackadaisical', "ladies' room", 'lager frenzy', 'laid out in lavender', 'la-la land', 'lamb to the slaughter', 'lame duck', 'land of nod', 'lardy-dardy', 'lark about', 'last but not least', 'last straw', 'latin phrases', 'laugh like a drain', 'laughing-stock', 'laurels', 'law is an ass', 'lay it on with a trowel', 'lead balloon', 'lead-pipe cinch', 'learn the ropes', "left hand doesn't know what the right hand is doing", 'left in the lurch', "legend in one's own lifetime", 'leopard cannot change its spots', 'less is more', 'let a thousand flowers bloom', 'let bygones be bygones', 'let not poor nelly starve', 'let not the sun go down on your wrath', 'let or hindrance', 'let sleeping dogs lie', 'let the cat out of the bag', 'let them eat cake', 'let there be light', 'let your hair down', "let's roll", 'level playing field', 'lick and a promise', 'lick into shape', 'lickety-split', 'lie low', 'lies, damned lies and statistics', 'life begins at forty', 'life in the 1500s - folk etymologies', 'life of riley', "life's not all beer and skittles", "life's too short", 'like a chicken with its head cut off', 'like a moth to a flame', 'like being savaged by a dead sheep', 'like billy-o', 'like it or lump it', 'like the clappers', 'like the dickens', 'like two peas in a pod', 'like turkeys voting for christmas', 'lily-livered', 'little bird told me', 'little knowledge is a dangerous thing', 'little of what you fancy does you good', 'little pitchers have big ears', 'lions led by donkeys', 'live long and prosper', 'living daylights', 'living off the fat of the land', 'living on borrowed time', 'lo and behold', 'load of cobblers', 'load of codswallop', 'loaf of bread', 'local derby', 'lock stock and barrel', 'long in the tooth', 'look before you leap', 'loose cannon', 'loose lips sink ships', 'lose face', 'lose your marbles', 'love is blind', 'love that dare not speak its name', 'loved-up', 'lunatic fringe', 'mad as a hatter', 'mad as a march hare', 'magical realism', 'main chance', 'make a bee-line for', 'make a clean breast of it', "make a pig's ear of", 'make a virtue of necessity', 'make haste', 'make hay while the sun shines', "make him an offer he can't refuse", 'make my day', 'make no bones about', 'make your hair stand on end', 'mal de mer', 'man after my own heart', 'man does not live by bread alone', 'man who is his own lawyer has a fool for a client', "man's best friend", "man's inhumanity to man", 'many a little makes a mickle', 'many are called but few are chosen', 'many happy returns', 'many a true word is spoken in jest', "'march' phrases", "mare's nest", 'marital aid', 'market forces', 'marry in haste, repent at leisure', 'may you live in interesting times', 'mccoy - the real', 'mea culpa', 'meat and drink', 'meat and two veg', 'meet your waterloo', 'memory lane', 'men in suits', 'mend fences', 'ménage à trois', "men's evil manners live in brass; their virtues we write in water", 'merry christmas', 'method in my madness', 'mexican wave', 'mickey finn', 'middle for diddle', 'middle of the road', 'might and main', 'mightier than the sword', 'mighty oaks from little acorns grow', 'milk of human kindness', 'minced oaths', 'minced words', 'mind your ps and qs', "mind's ear", "mind's eye", 'ministering angel shall my sister be', 'mint condition', 'misery acquaints a man with strange bedfellows', 'misquotes', 'miss is as good as a mile', 'moaning minnie', 'mondegreens', "montezuma's revenge", "'money' phrases", 'money for old rope', 'monstrous regiment of women', 'monty - the full', 'moot point', 'morbid obesity', 'more bang for your buck', 'more fool you', 'more haste, less speed', 'more honoured in the breach than in the observance', 'more light!', 'morning person', 'mother country', 'motley crew', 'mouth-watering', 'movable feast', 'movers and shakers', 'moving finger writes', 'moving the goalposts', 'much ado about nothing', 'much of a muchness', 'mud', 'mull (something) over', 'multitude of sins', 'mumbo jumbo', "mum's the word", "murphy's law", 'music has charms to soothe the savage breast', "mutt's nuts", 'mutt and jeff', 'mutton dressed as lamb', 'my bad', 'my better half', 'my cup of tea', 'my cup runneth over', 'my giddy aunt', 'my husband and i', "my mind's eye", 'my old china', 'my old dutch', 'my salad days', 'my stars and garters', 'my way or the highway', 'nail your colours to the mast', 'namby-pamby', 'name and shame', 'name is mud', 'nasty, brutish and short', 'nation of shopkeepers', 'nautical phrases', 'near the knuckle', 'necessity is the mother of invention', "ne'er cast a clout till may be out", "ne'er do well", 'needs must', 'neither a borrower nor a lender be', 'neither fish nor flesh, nor good red herring', 'neither here nor there', 'nest-egg', 'nest of vipers', 'never give a sucker an even break', 'never knowingly undersold', 'never look a gift horse in the mouth', 'never-never land', 'never the twain shall meet', 'new kid on the block', 'nice as ninepence', 'nice kettle of fish', 'nick of time', 'night owl', "nine days' wonder", 'nip and tuck', 'nip in the bud', 'nitty-gritty', 'no-brainer', 'no dice', 'no holds barred', 'no laughing matter', 'no love lost', 'no man is an island', 'no more cakes and ale?', 'no names: no pack-drill', 'no quarter given', 'no rest for the wicked', 'no room to swing a cat', 'no truck with', 'no way, jose', 'nod is as good as a wink', 'nod', 'norange', 'nosy parker', 'not a dicky-bird', 'not by a long chalk', 'not my cup of tea', 'not for all the tea in china', 'not playing with a full deck', 'not rocket science', 'not tonight josephine', 'not worth a plugged nickel', 'not worth the candle', 'nothing is certain but death and taxes', 'nothing succeeds like success', 'nothing ventured, nothing gained', 'now is the winter of our discontent', 'nth degree', 'nul points', 'nutshell', 'o romeo, romeo! wherefore art thou romeo?', 'o ye, of little faith', 'odds bodkins', 'off his own bat', 'off the record', 'off with his head', 'offing - in the', 'oh, my stars and garters', 'oh, they have slain the earl of moray and lady mondegreen', 'okay', 'okey-dokey', 'old as methuselah', 'old as the hills', 'old chestnut', 'old codger', 'old fogey', 'old hat', 'auld lang syne', 'on a hiding to nothing', 'on a wing and a prayer', 'on carey street', 'on cloud nine', 'on queer street', 'on tenterhooks', 'on the ball', 'on the bubble', 'on the button', 'on the dole', 'on the fiddle', "on the pig's back", 'on the qt', 'on the shoulders of giants', 'on the side of the angels', 'on the wagon', 'on the warpath', 'on with the motley', 'on your beam ends', 'on your tod', 'once bitten, twice shy', 'once in a blue moon', 'once more unto the breach, dear friends, once more', 'one-hit wonder', 'one fell swoop', 'one for the road', 'one foot in the grave', 'one good turn deserves another', 'one hand washes the other', 'one over the eight', 'one sandwich short of a picnic', 'one small step for man', 'one stop shop', "one swallow doesn't make a summer", "one's heart's content", 'oops-a-daisy', 'open season', 'order of the boot', 'ough - the many ways to pronounce', 'out of sight', 'out of sight, out of mind', 'out of sorts', 'out of the frying pan into the fire', 'out of the jaws of death', 'out of the mouths of babes and sucklings', 'out on a limb', 'over a barrel', 'over-egg the pudding', 'over the moon', 'over the top', 'overpaid, oversexed, and over here', 'oxo cube', 'ps and qs', 'packed to the gunwales', 'paddle your own canoe', 'paint the town red', 'panic stations', 'paper tiger', 'parting shot / parthian shot', 'pass over to the other side', 'pass the buck', 'paying guest - a', 'p.d.q. - pretty damn quick', 'pearls before swine', 'pears for your heirs', 'pecking order', 'peeping tom', 'peg out', 'pell-mell', 'pen is mightier than the sword', 'pennies from heaven', 'penny dreadful', 'penny drops', 'penny pinching', 'penny for your thoughts', 'penny saved is a penny earned', 'penny wise and pound foolish', 'pester power', 'peter out', 'petty cash', 'phrases and sayings quiz', 'phrases that people get wrong', 'physician heal thyself', "pick 'n' mix", 'picture is worth a thousand words', 'picture-perfect', 'pie', 'pie in the sky', 'piece of cake', 'piece of piss', 'piece of the action', 'pig and whistle', 'pig in a poke', 'pigeon-chested', "pig's back", "pig's ear", "pig's eye", 'piggyback', 'pigs might fly', 'pin money', 'pinch of salt', 'pip', 'pipe down', 'pipe dream', 'piping hot', 'piss and vinegar', 'pitch black', 'pitch dark', 'place for everything and everything in its place', 'plague on both your houses', 'plain sailing', 'plates of meat', 'play by ear', "play devil's advocate", 'play ducks and drakes', 'play fast and loose', 'play gooseberry', 'play it again sam', 'play second fiddle', 'play silly buggers', 'play the giddy goat', 'play the race card', 'pleased as punch', 'plug-ugly', 'plugged nickel', 'poetic justice', 'point-and-click', 'point percy at the porcelain', 'politically correct', 'point-blank', 'point to point', 'pommy bashing', 'pomp and circumstance', 'pond life', 'pony and trap', 'pony up', 'pop goes the weasel', 'pop your clogs', 'popular fallacies', 'pork pies', 'posh - port out, starboard home', 'pot calling the kettle black', 'potluck', 'pound of flesh', 'pour oil on troubled waters', 'power corrupts; absolute power corrupts absolutely', 'power dressing', 'powers that be', 'praying at the porcelain altar', 'preaching to the choir', 'preposterous', 'press into service', 'pretty kettle of fish', 'pretty penny', 'prick up your ears', 'pride comes before a fall', 'prime time', 'primrose path', 'procrastination is the thief of time', 'proof of the pudding is in the eating', 'proverbs', 'pull in your horns', 'pull out all the stops', "pull the other one, it's got bells on", 'pull the wool over your eyes', 'pull up stakes', 'pull your horns in', 'pull your finger out', 'pull yourself up by your bootstraps', "pulling one's leg", "punch above one's weight", 'pure as the driven snow', 'purple patch', 'push the boat out', 'push the envelope', 'put a damper on', 'put a sock in it', 'put a spanner in the works', 'put on the wooden overcoat', 'put on your thinking cap', 'put paid to', 'put the cart before the horse', 'put the mockers on', 'put the wood in the hole', 'put up your dukes', 'put your back up', 'put your best foot forward', 'put your nose out of joint', 'put your shoulder to the wheel', 'pyrric victory', 'quality time', 'quantum leap', 'quarter', 'queer as a nine bob note', 'queer street', 'queer the pitch', 'quick and the dead', 'quicker than lager turns to piss', 'quid pro quo', 'quotations', 'rabbit and pork', 'rack and ruin', 'rag-and-bone man', 'rack your brains', 'rag, tag and bobtail', 'raining cats and dogs', 'raining stair-rods', 'raise cain', 'rank and file', 'raspberry tart', 'raze to the ground', 'rat arsed', 'read between the lines', 'read the riot act', 'real mccoy', 'red-handed', 'red herring', 'red in tooth and claw', 'red-letter day', 'red rag to a bull', 'red sky at night ...', 'red tape', 'reduplicated phrases', 'religion is the opium of the people', 'rest on his laurels', 'revenge is a dish best served cold', 'rhyme nor reason', 'rhyming slang', 'richard of york gave battle in vain', 'richard the third', 'riddle wrapped up in an enigma', 'ride a cock-horse to banbury cross', 'riding shotgun', 'riff-raff', 'riley', 'rings a bell', "ring a ring o'roses, a pocketful of posies, atishoo, atishoo, all fall down", 'ring down the curtain', 'ring-fencing', 'ring the changes', 'rinky-dink', 'rise and shine', 'road less travelled', 'road apples', 'road rage', 'roasted to a turn', 'rob peter to pay paul', 'rock and roll', 'rocket science', 'a rolling stone gathers no moss', 'romeo, romeo, wherefore art thou romeo?', 'rose by any other name would smell as sweet', 'rose is a rose is a rose', 'rosie lea', 'rough diamond', 'round robin', 'route one', 'rub of the green', 'rule of thumb', 'rum do', 'rumpy-pumpy', 'run a mile', 'run amok', 'run of the mill', 'run out of steam', 'run rings around', 'run the gauntlet', 'sacred cow', 'safe as houses', 'safe pair of hands', 'safe sex', 'salad days', 'salt of the earth', 'san fairy ann', 'sandwich short of a picnic', 'save face', "save one's bacon", 'saved by the bell', 'savoir faire', 'say cheese', 'say goodnight gracie', 'scapegoat', 'scarper', 'scot-free', 'scott', 'screw your courage to the sticking place', "'scuse me while i kiss this guy", 'sea change', 'sealed with a loving kiss', 'season of mists and mellow fruitfulness', 'second-guess', 'security blanket', "see a pin and pick it up, all the day you'll have good luck; see a pin and let it lie, bad luck you'll have all day", 'see red', 'seek and you shall find', 'seen better days', 'send packing', 'senior citizen', 'sent to coventry', 'separate the sheep from the goats', "set one's cap at", 'set your teeth on edge', 'sex and shopping', 'seven-year itch', 'sexton blake', 'shebang', 'shaggy dog story', 'shake a leg', 'shakers and movers', 'phrases coined by william shakespeare', 'words coined by william shakespeare', "shall i compare thee to a summer's day?", 'shambles', "shanks' mare/shanks' pony", 'share and share alike', 'shilly-shally', 'ship shape and bristol fashion', 'shit end of the stick', 'shit for brains', 'shiver my timbers', 'shoddy', 'short end of the stick', 'shot across the bows', "shot heard 'round the world", 'shot in the arm', 'shot in the dark', 'shoot through', 'short shrift', 'show a leg', 'show your mettle', 'shrinking violet', 'shuffle off this mortal coil', 'shut your cake-hole', 'siamese twins', 'sick puppy', 'sight for sore eyes', 'silly billy', 'silly season', 'silver lining', 'sir walter scott', 'silence is golden', 'silver bullet', 'similes', 'sitting pretty', 'sixes and sevens', 'skeleton in the closet', 'skid row', 'skin and blister', 'skin of your teeth', "sky's the limit", 'sledgehammer to crack a nut', 'sleep like a top', 'sleep on a clothesline', 'sleep tight', 'sleeveless errand', 'sloane ranger', 'slush fund', 'smallest room in the house', 'smart casual', 'smoke and mirrors', 'snug as a bug in a rug', 'so sue me', 'soap-dodger', "sod's law", 'sold down the river', "some are born great, some achieve greatness, and some have greatness thrust upon 'em", 'some men are born mediocre, some men achieve mediocrity, and some men, have mediocrity thrust upon them', 'someone is walking over my grave', 'something for the weekend sir?', 'something nasty in the woodshed', 'something old, something new, something borrowed, something blue', 'son of a bitch', 'son of a gun', 'sorry sight', 'sound bite', 'sound out', 'sour grapes', 'space, the final frontier', 'spare the rod and spoil the child', 'speak of the devil', 'speak softly and carry a big stick', 'special relationship', 'spelling-bee', 'spend a penny', 'spick and span', 'spill the beans', 'spin doctor', 'spitting feathers', 'spitting image', 'spoonerisms', 'spring forward, fall back', 'spruce-up', 'square meal', 'squeaky bum time', 'stand and deliver', 'stand up guy', 'standing on the shoulders of giants', 'star-crossed lovers', 'stark, raving mad', 'stars and garters', 'start from scratch', 'steal a march', 'steal my thunder', 'step up to the plate', 'stick in the mud', 'sticks and stones may break my bones', 'sticky wicket', 'stiff upper lip', 'stiffen the sinews', 'stinking rich', 'stitch in time saves nine', 'stone the crows', "stone's throw", 'stony-hearted', 'stool pigeon', 'storm in a teacup', 'straight as a die', "straight from the horse's mouth", 'strain at the leash', 'strait-laced', 'strait and narrow', 'stranger danger', 'stranger than fiction', 'strike while the iron is hot', 'stuff and nonsense', 'stump up', 'such is life', 'surf and turf', 'surfing the net/surfing the internet/surfing the web', 'survival of the fittest', 'suspension of disbelief', 'swan song', 'sweet fanny adams', 'swing for you', 'swing the lead', 'tail wagging the dog', 'take a back seat', 'take care of the pence and the pounds will take care of themselves', 'take down a peg or two', 'take potluck', 'take the bit between your teeth', 'take the cake', 'take the gilt off the gingerbread', 'take the mickey', 'take the upper hand', 'take umbrage', 'take with a grain of salt', 'taken aback', 'taken for a ride', 'talk of the devil', 'talk the talk', "talk through one's hat", 'talk to the hand', 'tall story', "tanstaafl - (there's no such thing as a free lunch)", 'tattoo phrases and sayings', 'tawdry', 'tea leaf', 'technicolor yawn', 'tell it to the marines', 'tell me about it', 'tempest in a teapot', 'show your mettle', "that's all folks!", "that's all she wrote", "that's one small step for a man, a giant leap for mankind", "the ants are my friends, they're blowing in the wind", 'the apple never falls far from the tree', 'the apple of my eye', 'the back of beyond', 'the balance of power', 'the balance of trade', 'the bane of my life', "the bee's knees", 'the belle of the ball', 'the best defense is a good offence', 'the best laid schemes of mice and men', 'the big apple', 'the big easy', 'the birds and the bees', 'the bitter end', 'the blind leading the blind', 'the bread of life', 'the buck stops here', 'the call of the wild', 'the camera cannot lie', 'the chickens come home to roost', 'the child is father to the man', 'the collywobbles', 'the course of true love never did run smooth', 'the crack of doom', 'the crapper', 'the customer is always right', 'the cut of your jib', 'the dark side', 'the darkest hour is just before the dawn', 'the devil has all the best tunes', 'the devil incarnate', 'the devil is in the details', 'the devil makes work for idle hands to do', 'the devil take the hindmost', 'the devil to pay', 'the die has been cast', 'the early bird catches the worm', 'the elephant in the room', "the emperor's new clothes", 'the ends of the earth', 'the exception which proves the rule', 'the face that launched a thousand ships', 'the fat of the land', 'the female of the species is more deadly then the male', 'the fifth estate', 'the floozie in the jacuzzi', 'the fly in the ointment', 'the full monty', 'the game is afoot', 'the game is up', 'the great unwashed', 'the hair of the dog that bit you', 'the hairy eyeball', 'the heebie-jeebies', 'the jury is still out', 'the land of nod', 'the last straw', 'the law is an ass', 'the life of riley', 'the living daylights', 'the love that dare not speak its name', 'the monstrous regiment of women', 'the more the merrier', 'the moving finger writes', "the mutt's nuts", 'the nth degree', "the opera ain't over till the fat lady sings", 'the pen is mightier than the sword', 'the pip', 'the pot calling the kettle black', 'the powers that be', 'the proof of the pudding is in the eating', 'the quick and the dead', 'the real mccoy', 'the road less travelled', 'the road to hell is paved with good intentions', 'the salt of the earth', 'the seven-year itch', 'the shoemaker always wears the worst shoes', 'the short end of the stick', "the shot heard 'round the world", "the sky's the limit", 'the smallest room in the house', "the straw that broke the camel's back", 'the tail wagging the dog', 'the toast of the town', 'the third degree', 'the triumph of hope over experience', 'the upper hand', 'the usual suspects', 'the whole kit and caboodle', 'the whole nine yards', 'the whole shebang', 'the writing is on the wall', 'the wrong side of the blanket', 'the year dot', 'the yellow peril', 'there is more than one way to kill a cat', 'there are three kinds of lies...', 'there but for the grace of god, go i', 'there is no alternative', "there's an r in the month", "there's method in my madness", "there's no fool like an old fool", "there's no place like home", "there's no such thing as a free lunch (tanstaafl)", "there's no such thing as bad publicity", "there's one (a sucker) born every minute", "there's one law for the rich and another law for the poor", "they couldn't hit an elephant at this distance", 'thick and fast', 'thick and thin', 'thick as thieves', 'thick as two short planks', 'thin air', 'thing of beauty is a joy forever', 'things that go bump in the night', 'think outside the box', 'thinking cap', 'third degree', 'third time lucky', 'thomas paine', 'thorn in the flesh', 'thou shalt not kill', 'though this be madness, yet there is method in it', 'three score and ten', 'three sheets to the wind', 'three strikes and you are out', 'through thick and thin', 'throw in the towel', 'throw good money after bad', 'throw your hat into the ring', 'thumbs up', 'thus far into the bowels of the land', 'ticked off', 'tickle the ivories', 'tickled pink', 'tide over', 'tie the knot', 'till the cows come home', 'time and tide wait for no man', "tinker's damn", 'tilting at windmills', 'tissue of lies', 'titfer', 'tit for tat', 'tits up', 'to a t', 'to all intents and purposes', 'to be or not to be, that is the question', 'to beggar belief', 'to boldly go where no man has gone before', 'to boot', 'to cast the first stone', 'to err is human; to forgive divine', 'to gild refined gold, to paint the lily', "to sleep: perchance to dream: ay, there's the rub", 'to the manner born', 'to the nth degree', 'to travel hopefully is a better thing than to arrive', 'toast of the town', 'toe the line', 'toe-curling', 'toffee-nosed', 'tommy atkins', 'tomorrow is another day', 'tongue in cheek', 'too big for your breeches', 'too much of a good thing', 'toodle-oo', 'tooth and nail', 'top dog', 'top drawer', 'top notch', 'topsy-turvy', 'total shambles', 'touch and go', 'touch with a barge-pole', 'touch wood', 'touchy-feely', 'tout de suite', 'tower of strength', 'train surfing', 'trick or treat', 'trip the light fantastic', 'trouble and strife', 'true blue', 'truth is stranger than fiction', 'truth will out', 'tuckered out', 'turkeys voting for christmas', 'turn a blind eye', 'turn of phrase', 'turn the tables', 'turn up for the books', 'turn up trumps', 'twelve good men and true', 'twenty three skidoo', 'twenty four seven', 'two cents worth', 'two heads are better than one', 'two peas in a pod', 'under the thumb', 'under your hat', 'under the auspices of', 'underway', 'under wraps', 'uneasy lies the head that wears a crown', 'union jack', 'unwashed', 'unkindest cut of all', 'up a blind alley', 'up a gum tree', 'up in arms', 'up the creek without a paddle', 'up the apples and pears', 'up the duff', 'up the pole', 'up the wooden hill to bedfordshire', 'up to snuff', 'upper crust', 'upper hand', 'up in arms', 'ups-a-daisy', 'upset the apple-cart', 'up the ante', 'upside down', 'urban myth', 'use your loaf', 'usual suspects', 'using a sledgehammer to crack a nut', "valentine's day poems and sayings", 'van surfing', 'vanish into thin air', 'veg out', 'verbosity leads to unclear, inarticulate things', 'vice versa', 'vicious circle', 'vis-a-vis', "visit the ladies' room", 'vorsprung durch technik', 'walk free', 'walk the walk', 'walk the plank', 'walter scott', 'warts and all', 'watch the birdie', 'watched pot never boils', 'watching brief', 'wave a red rag to a bull', 'wax lyrical', 'wax poetic', 'we are a grandmother', 'we are not amused', 'we few, we happy few, we band of brothers', 'we have seen better days', 'we know where you live', 'weakest link', 'wear the trousers', 'wear your heart on your sleeve', 'weasel words', 'wee-wee', 'well heeled', 'well read', 'wet behind the ears', 'wet blanket', 'what a piece of work is man', 'what are you like?', 'what football is all about', 'what god has joined together let no man put asunder', "what part of no don't you understand?", 'what the dickens', 'what you see is what you get - wysiwyg', "what's in a name? that which we call a rose by any other name would smell as sweet", "what's not to like?", "what's up doc?", 'when in rome, do as the romans do', 'when it comes to the crunch', 'when pigs fly', 'when the going gets tough, the tough get going', 'when the shit hits the fan', "where there's muck there's brass", 'whet your appetite', 'which is which?', 'which witch is which?', 'while you live, tell truth and shame the devil!', 'whip round', 'whipper snapper', 'whipping boy', 'whistle and flute', 'whistle blower', 'whistle down the wind', 'white as snow', 'white bread', 'white elephant', 'whole kit and caboodle', 'whole-hearted', 'whole nine yards', 'whole shebang', 'whoops-a-daisy', 'why does bread always fall butter side down?', 'why should the devil have all the best tunes?', 'wick', 'wide berth', 'wild and woolly', 'wild goose chase', 'willy nilly', 'win hands down', 'win one for the gipper', 'wing and a prayer', 'wing it', 'winter draws on', 'wish you were here', 'with bells on', 'without a shadow of a doubt', 'without let or hindrance', 'without so much as a by your leave', 'woe betide you', 'woe is me', "wolf in sheep's clothing", 'woman needs a man like a fish needs a bicycle', "woman's place is in the home", 'women and children first', 'wooden hill to bedfordshire', 'word association football', 'word for word', 'word in edgeways', 'word in your shell-like', 'words ending in gry', 'worse for wear', "worth one's salt", 'wotcher', "wouldn't touch with a barge-pole", 'wreak havoc', 'writing is on the wall', 'wrong end of the stick', 'wrong side of the blanket', 'yada yada', 'year dot', 'yellow-belly', 'yellow peril', 'you are what you eat', 'you can catch more flies with honey than with vinegar', "you can lead a horse to water but you can't make it drink", "you can lead a whore to culture but you can't make her think", "you can't get blood out of a stone", "you can't have your cake and eat it too", "you can't hold with the hare and run with the hounds", "you can't make a silk purse out of a sow's ear", "you can't teach an old dog new tricks", 'you get on my wick', "you look as if you've been dragged through a hedge backwards", "you'll wonder where the yellow went when you brush your teeth with pepsodent", 'young turk', 'your days are numbered', 'your money or your life', 'your name is mud', "you've never had it so good", 'zero tolerance', 'zig-zag'])


IDIOMS.extend(['adoption authority of ireland (aai)', 'an bord pleanála', 'an garda síochána', 'aquaculture licences appeals board (alab)', 'arts council (an chomhairle ealaíon)', 'athlone institute of technology', 'bord bia (irish food board)', 'bord iascaigh mhara (irish sea fisheries board)', 'broadcasting authority of ireland (bai)', 'cavan and monaghan education and training board', 'central bank of ireland', 'central statistics office', 'chester beatty library', 'chief state solicitor’s office', 'child and family agency (tusla)', 'chomhairle um oideachais gaeltachta agus gaelscolaíochta', 'citizens information board', 'city of dublin education and training board', 'commission for aviation regulation', 'commission for communications regulation (ccr comreg)', 'commission for public service appointments', 'commission for railway regulation', 'commission for regulation of utilities (cru)', 'commission to inquire into child abuse', 'commissioner for environmental information', 'commissioners of irish lights', 'competition and consumer protection commission (ccpc)', 'cork education and training board', 'cork institute of technology', 'crawford art gallery', 'data protection commission', 'defence forces of ireland', 'dental council', 'director of public prosecutions', 'discovery programme', 'donegal education and training board', 'dublin and dun laoghaire education and training board', 'dublin institute for advanced studies', 'dublin institute of technology', 'dundalk institute of technology', 'eastern and midland regional assembly', 'educational research centre (erc)', 'enterprise ireland', 'environmental protection agency (epa)', 'financial services and pensions ombudsman (fspo)', 'food safety authority of ireland (fsai)', 'food safety promotion board (safefood)', 'foras na gaeilge - an foras teanga (the language body)', 'forensic science ireland', 'fáilte ireland', "gaisce (president's national challenge award)", 'galway and roscommon education and training board', 'galway-mayo institute of technology', 'garda ombudsman', 'grangegorman development agency', 'health and safety authority', 'health and social care professionals council', 'health information and quality authority (hiqa)', 'health insurance authority', 'health products regulatory authority (hpra)', 'health research board (hrb)', 'health service executive', 'health service national partnership forum (lenus)', 'healthy ireland', 'higher education authority (hea)', 'horse racing ireland (hri)', 'horse sport ireland', 'housing agency', 'housing finance agency', 'ida ireland', 'inland fisheries ireland', 'insolvency service ireland', 'institute of art, design and technology, dun laoghaire', 'institute of public health in ireland (iph)', 'institute of technology blanchardstown', 'institute of technology carlow', 'institute of technology sligo', 'institute of technology tallaght', 'institute of technology tralee', 'irish auditing and accounting supervisory authority', 'irish blood transfusion service board (ibts)', 'irish film classification office', 'irish financial services appeals tribunal', 'irish fiscal advisory council', 'irish heritage trust', 'irish human rights and equality commission', 'irish manuscripts commission', 'irish museum of modern art (imma)', 'irish prison service', 'irish rail', 'irish research council', 'irish water', 'irish water safety', 'kerry education and training board', 'kildare and wicklow education and training board', 'kilkenny and carlow education and training board', 'laois and offaly education and training board', 'legal aid board', 'legal services regulatory authority', 'letterkenny institute of technology', 'limerick and clare education and training board', 'limerick institute of technology', 'local government management agency (lgma)', 'longford and westmeath education and training board', 'louth and meath education and training board', 'léargas (exchange bureau)', 'marine casualty investigation board', 'marine institute (foras na mara)', 'mayo, sligo and leitrim education and training board', 'medical bureau of road safety', 'medical council', 'mental health (criminal law) review board', 'mental health commission', 'moorepark technology ltd', 'nama-national asset management agency', 'national archives of ireland', 'national cancer registry board', 'national centre for guidance in education', 'national council for curriculum and assessment', 'national council for special education', 'national development finance agency (ndfa)', 'national disability authority', 'national economic and social council', 'national gallery of ireland', 'national library of ireland (nli)', 'national milk agency', 'national museum of ireland (nmi)', 'national oil reserves agency (nora)', 'national property services regulatory authority', 'national shared services office', 'national standards authority of ireland (nsai)', 'national transport authority', 'national treasury management agency (ntma)', 'national treatment purchase fund board', 'northern and western regional assembly', 'nursing and midwifery board of ireland', 'oberstown children detention campus', 'office of emergency planning', 'office of government procurement', 'office of ombudsman for children', 'office of ombudsman for the defence forces', 'office of public works', 'office of the attorney general', 'office of the comptroller and auditor general', 'office of the information commissioner', 'office of the ombudsman', 'office of the planning regulator', 'office of the refugee appeals tribunal', 'office of the revenue commissioners', 'oifig choimisinéir na dteangacha oifigúla', 'ordnance survey ireland', 'pensions council', 'pharmaceutical society of ireland', 'pobal', 'pre-hospital emergency care council (phecc)', "president's establishment", 'private security authority', 'property registration authority', 'public appointments service', 'quality and qualifications ireland', 'regulator of the national lottery', 'residential institutions redress board (rirb)', 'residential institutions statutory fund board (caranua)', 'residential tenancies board (prtb)', 'road safety authority', 'rté – raidió teilifís éireann', 'safety on the water', 'science foundation ireland', 'screen ireland (fís éireann)', 'sea fisheries protection authority (sfpa)', 'skillnet ireland', 'social welfare appeals office', 'solas', 'southern regional assembly', 'sport ireland', 'standards in public office commission', 'state examinations commission (sec)', 'strategic banking corporation of ireland', 'sustainable energy authority of ireland (seai)', 'tax appeals commission', 'teaching council', 'teagasc (agriculture and food development authority)', 'teilifís na gaeilge - tg4', 'the charities regulator', 'the courts service', 'the heritage council', 'the international protection office', 'the loughs agency', 'the national oversight and audit commission', 'the pensions authority', 'the state laboratory', 'the water advisory body', 'tipperary education and training board', 'tourism ireland', 'trade and business development body (intertradeireland)', 'transport infrastructure ireland', 'ulster-scots agency', 'valuation office', 'water safety', 'waterford and wexford education and training board', 'waterford institute of technology', 'waterways ireland', 'western development commission (wdc)', 'workplace relations commission', 'údarás na gaeltachta'])

CUSTOM_PHRASES = ['will be', 'get things done', 'local involvement', 'value for money', 'jump to conclusions', 'grounds for', 'do not like', 'on the record', 'off the record', 'on his behalf', 'on her behalf', 'well spent', 'no more', 'no longer', 'round of discussions', 'hours a day', 'gone too far', 'too much to lose', 'health issues', 'health issue', 'word games', 'word game', 'matter of fact', 'evil deeds', 'tax evasion', 'take some time', 'red light', 'green light', 'trade unions', 'trade union', 'local government', 'tax avoidance', 'to be fair']


IDIOMS.extend(CUSTOM_PHRASES)

for idiom in IDIOMS.copy():
    IDIOMS.append(replace_contractions(idiom))
    IDIOMS.append(replace_contractions(idiom.capitalize()))

IDIOMS = list(set(IDIOMS))


IDIOM_MATCHER = Matcher(ENCORE.vocab)


for idiom in IDIOMS:
    IDIOM_MATCHER.add(
        'QBF',
        None,
        [
            {
                'LOWER': i.lower()
            } for i in REPLACEMENTS
        ]
    )


for idiom in IDIOMS:
    IDIOM_MATCHER.add(
        'QBF',
        None,
        [
            {
                'LOWER': i.lower()
            } for i in idiom.split(' ')
        ])
